summaryrefslogtreecommitdiffstats
path: root/system
diff options
context:
space:
mode:
authorAndrey Andreev <narf@devilix.net>2022-01-05 20:42:05 +0100
committerAndrey Andreev <narf@devilix.net>2022-01-05 20:42:05 +0100
commit2afffc07780f29767d8b212f7c2cbde4ab56ef51 (patch)
treebcbc1f7ac2219d11d35a1e84db844cb197c8c792 /system
parent29d07cb116a31ebae58266f05c3c2eac3eeccb08 (diff)
parentf5255269f21e66481166518db2653dc3e8c46202 (diff)
Merge branch 'feature/uncache_ci_vars' of github.com:TalonTR/Codeigniter into feature/viewvars
Diffstat (limited to 'system')
-rw-r--r--system/core/Loader.php51
1 files changed, 46 insertions, 5 deletions
diff --git a/system/core/Loader.php b/system/core/Loader.php
index d9a1539aa..fb8b73b1f 100644
--- a/system/core/Loader.php
+++ b/system/core/Loader.php
@@ -94,6 +94,13 @@ class CI_Loader {
protected $_ci_cached_vars = array();
/**
+ * Stack of variable arrays to provide nested _ci_load calls all variables from parent calls
+ *
+ * @var array
+ */
+ protected $_ci_load_vars_stack = array();
+
+ /**
* List of loaded classes
*
* @var array
@@ -934,15 +941,46 @@ class CI_Loader {
}
/*
* Extract and cache variables
*
* You can either set variables using the dedicated $this->load->vars()
* function or via the second parameter of this function. We'll merge
- * the two types and cache them so that views that are embedded within
- * other views can have access to these variables.
+ * the two types so that loaded views and files have access to these
+ * variables.
+ * Additionally we want all subsequent nested _ci_load() calls embedded
+ * within the current file to 'inherit' all variables that are
+ * accessible to the current file. For this purpose we push the current
+ * variable configuration (_ci_vars) to the stack and remove it again
+ * after the file or view is completely loaded. Nested _ci_load() calls
+ * within the current file extend the stack with their variable
+ * configuration.
*/
- empty($_ci_vars) OR $this->_ci_cached_vars = array_merge($this->_ci_cached_vars, $_ci_vars);
- extract($this->_ci_cached_vars);
+
+ // Init current _ci_vars as current variable configuration
+ if ( ! is_array($_ci_vars))
+ {
+ $_ci_vars = [];
+ }
+
+ // Include the global cached vars into the current _ci_vars if needed
+ if ( ! empty($this->_ci_cached_vars))
+ {
+ $_ci_vars = array_merge($this->_ci_cached_vars, $_ci_vars);
+ }
+
+ // Merge the last variable configuration from a parent _ci_load()
+ // call into the current _ci_vars
+ if ( ! empty($this->_ci_load_vars_stack))
+ {
+ $previous_variable_configuration = end($this->_ci_load_vars_stack);
+ $_ci_vars = array_merge($previous_variable_configuration, $_ci_vars);
+ }
+
+ // Push the current _ci_vars to the stack
+ array_push($this->_ci_load_vars_stack, $_ci_vars);
+
+ // Extract the current _ci_vars
+ extract($_ci_vars);
/**
* Buffer the output
@@ -960,6 +998,9 @@ class CI_Loader {
include($_ci_path); // include() vs include_once() allows for multiple views with the same name
log_message('info', 'File loaded: '.$_ci_path);
+ // Remove current _ci_vars from stack again
+ array_pop($this->_ci_load_vars_stack);
+
// Return the file data if requested
if ($_ci_return === TRUE)
{
@@ -934,15 +941,46 @@ class CI_Loader {
}
/*
- * Extract and cache variables
+ * Extract and stack variables
*
* You can either set variables using the dedicated $this->load->vars()